import venv, os
from presets import VIRTUALENV_DIR, PACKAGE_INSTALL_COMMAND

def create_virtual_env(directory, install_libs=True):
    builder = venv.EnvBuilder(with_pip=True, upgrade_deps=True)
    builder.create(os.path.join(os.getcwd(), directory, VIRTUALENV_DIR))

    if install_libs:
        # Attempt to enter the virtual environment and install required scripts there.
        # Might be tricky since Windows and Linux differ in activating the virtualenv
        
        code = os.system(PACKAGE_INSTALL_COMMAND.format(name=directory))
        if code == 0:
            print('\tDjango installed successfully')
            return
        print('\tCouldn\'t install Django. The following steps may fail')

    


import argparse, os
from datetime import datetime as dt
from misc import create_directory
from virtenv import create_virtual_env
from add_git import initialize_repo
from django_proj import create_django_project
from docker_files import wrap

if __name__ != '__main__':
    print('This file is to be executed directly')
    raise SystemExit

parser = argparse.ArgumentParser(description="Creates a Django project with virtual environment, Git repository and Docker container")
parser.add_argument('name', type=str, help="Name of your project")
parser.add_argument('-r', '--repository', type=str, help="Link to the repository")
args = parser.parse_args()

if os.path.exists(os.path.join(os.getcwd(), args.name)):
    print(f'\nTarget directory "{os.path.join(os.getcwd(), args.name)}" is not empty. Operation aborted\n')
    raise SystemExit

print(f'[{dt.now()}] Initializing "{args.name}"...')

create_directory(args.name) # Create directory
print('\tProject folder created')

if args.repository:
    initialize_repo(args.name, args.repository) # Initialize the repository and pull it
    print('\tGit repository initialized')
else:
    print('\tSkipping initializing Git - no link provided')

create_virtual_env(args.name) # Create a virtual environment in this directory
print('\tVirtual environment created')

create_django_project(args.name) # Initialize a Django project 
print('\tDjango project created')

wrap(args.name) # Create Dockerfile and docker-compose.yml file
print('\tWrapped in Docker')







DJANGO_PROJECT_CREATE_COMMAND = 'cd {name} && cd venv && cd Scripts && "activate" && cd ../.. && django-admin startproject {name} .'
VIRTUALENV_DIR = 'venv'
PACKAGE_INSTALL_COMMAND = 'cd {name} && cd venv && cd Scripts && "activate" && pip install django && cd ../../ && pip freeze > requirements.txt'


import os

def create_directory(dir, raiseFileExists=False):
    try:
        os.mkdir(dir)
    except FileExistsError:
        if raiseFileExists:
            raise
    except OSError:
        raise

def get_script_location():
    """ 
    Returns the path to current file
    """
    return os.path.dirname(os.path.realpath(__file__))


import os
from misc import get_script_location

def wrap(name):
    with open(os.path.join(os.getcwd(), name, 'Dockerfile'), 'w+') as t, open(os.path.join(get_script_location(), 'docker', 'Dockerfile'), 'r') as s:
        t.write(s.read())

    with open(os.path.join(os.getcwd(), name, 'docker-compose.yml'), 'w+') as t, open(os.path.join(get_script_location(), 'docker', 'docker-compose.yml'), 'r') as s:
        t.write(s.read())